# OctaneRender for Poser: Compress Texture Maps
# 
# Copyright (c) 2012, Paul Kinnane, Physicalc Software Pty Ptd
#
# For additional support, contact Paul Kinnane at support@physicalc-software.com
#
# Version 1.0 - original release
#
# 1.28.0.19 - Compressed textures now stored in the LiveDb folder
#
# 1.20.2.3 - Changed texture location from Poser Temp folder to the Poser Preferences folder
#
# 1.02s - Added Select All button
#
#========================================================================================================================================
import poser
import os
import sys
import wx
from xml import etree
import xml.etree.ElementTree as ET

m_LiveDbTexturemapFolder = ""

def LoadSettings():
    global m_LiveDbTexturemapFolder
    
    if os.path.exists(os.path.join(poser.PrefsLocation(),'config.xml')):
        try:
            tree = ET.parse(os.path.join(poser.PrefsLocation(),'config.xml'))    
            root = tree.getroot()
                
            m_LiveDbTexturemapFolder = root.get('LiveDbTexturemapFolder')
            if m_LiveDbTexturemapFolder == None or m_LiveDbTexturemapFolder == "":
                m_LiveDbTexturemapFolder = poser.PrefsLocation()    
                
        except:
            print "Error reading 'config.xml'"
            m_LiveDbTexturemapFolder = poser.PrefsLocation() 

class CompressTexturemaps(wx.Dialog): 

    def __init__(self):
        global finished
        finished = True
        
        FGCOLOR = wx.Colour(242,242,242)
        BGCOLOR = wx.Colour(74,74,74)

        DLGWIDTH = 430
        wx.Dialog.__init__(self, None, -1, "Compress Texturemaps", size=wx.Size(DLGWIDTH, 415))

        self.SetBackgroundColour(BGCOLOR)
        self.SetForegroundColour(FGCOLOR)

        col1X = 10
        col2X = 210
        col3X = col2X + 70
        col4X = col3X + 70
        col5X = col4X + 70
        y = 10
        yGap = 38

        wx.StaticText(self, label="Figure/Prop:", pos=(15, y))
        propList = []
        for figure in poser.Scene().Figures():
            propList.append(figure.Name())        
        for prop in poser.Scene().Actors():
            if prop.IsProp():
                if prop.Name().lower()[:8] not in ["focusdis", "goalcent", "centerof"]:
                    propList.append(prop.Name())
        self.propCB = wx.ComboBox(self, size=(290, -1), pos=(col2X - 100, y), choices=propList, style=wx.CB_DROPDOWN)
        self.Bind(wx.EVT_COMBOBOX, self.EventPropChange, self.propCB)
        self.propCB.SetStringSelection(propList[0])
        self.propCB.SetToolTip(wx.ToolTip("Select the figure or prop to compress texturemaps for."))

        y = y + yGap
        self.selectAllButton = wx.Button(self, label="Select All", pos=(15, y), size=(80, -1))
        self.Bind(wx.EVT_BUTTON, self.EventSelectAll, self.selectAllButton)

        self.checkListBox = wx.CheckListBox(self, size=(290, 200), pos=(col2X - 100, y))

        self.EventPropChange(None)
                
        y = y + yGap + 190
        wx.StaticText(self, label="Compress To Percent (1-99):", pos=(15, y))
        self.compressionTextCtrl = wx.TextCtrl(self, size=(50, -1), pos=(col2X, y - 4))
        self.compressionTextCtrl.Bind(wx.EVT_CHAR, self.EventTextCtrlKey)
        self.compressionTextCtrl.SetBackgroundColour(BGCOLOR)
        self.compressionTextCtrl.SetForegroundColour(FGCOLOR)
        self.compressionTextCtrl.SetValue("50")
        self.compressionTextCtrl.SetToolTip(wx.ToolTip("10 = texturemaps will be reduced to 1/10th of their current size.  50 = texturemaps will be halved in size."))

        y = y + yGap
        wx.StaticText(self, label="Total Memory Required:", pos=(15, y))
        self.totalMemoryStaticText = wx.StaticText(self, pos=(col2X, y))
                
        y = y + yGap
        self.compressButton = wx.Button(self, label="Compress", pos=(15, y), size=((DLGWIDTH / 2) - 30, -1))
        self.Bind(wx.EVT_BUTTON, self.EventCompress, self.compressButton)
        self.closeButton = wx.Button(self, label="Close", pos=(DLGWIDTH / 2, y), size=((DLGWIDTH / 2) - 30, -1))
        self.Bind(wx.EVT_BUTTON, self.EventClose, self.closeButton)
        
        self.printMem()
        
    def EventSelectAll(self, e):
        for i in range(self.checkListBox.GetCount()):
            self.checkListBox.Check(i)
        pass
        
    def fillMaterialsList(self, propFigure):
        self.SetCursor(wx.StockCursor(wx.CURSOR_WAIT))
        self.findAllImageFiles(propFigure)
        self.checkListBox.Clear()
        for m in self.imageFiles:
            fName = m
            if string.rfind(fName, "\\") != -1:
                fName = fName[string.rfind(fName, "\\") + 1:]
            if string.rfind(fName, "/") != -1:
                fName = fName[string.rfind(fName, "/") + 1:]
            nolog = wx.LogNull() 
            image = wx.Image(m)
            del nolog 
            size = int(max(image.GetWidth(), image.GetHeight()))
            #self.imageFiles.append(fName + " (" + str(size) + " x " + str(size) + ")")
            i = self.checkListBox.Append(fName + " (" + str(size) + " x " + str(size) + ")")
            self.checkListBox.SetClientData(i, m)
        self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))

    def findAllImageFiles(self, propFigure):
        self.imageFiles = []
        self.textureList = []
        self.bumpList = []
        
        self.TotalMemory = 0.0
        self.GlobalRGBFileList = []
        self.GlobalGrayscaleFileList = []       
        
        for material in propFigure.Materials():
            # Scan for the DIFFUSE map
            node = material.ShaderTree().Nodes()[0].InputByInternalName("Diffuse_Color").InNode()
            self.scanForTexture(node, "diffuse")
            node = material.ShaderTree().Nodes()[0].InputByInternalName("AlternateDiffuse").InNode()
            self.scanForTexture(node, "diffuse")

            # Scan for the BUMP map
            node = material.ShaderTree().Nodes()[0].InputByInternalName("Bump").InNode()
            self.scanForTexture(node, "bump")

            # Scan for the TRANS map
            node = material.ShaderTree().Nodes()[0].InputByInternalName("Transparency_Max").InNode()
            self.scanForTexture(node, "bump")
            
#            if m.TextureMapFileName() != None:
#                if not m.TextureMapFileName() in self.imageFiles:
#                    self.imageFiles.append(m.TextureMapFileName())
#
#            if m.TransparencyMapFileName() != None:
#                if not m.TransparencyMapFileName() in self.imageFiles:
#                    self.imageFiles.append(m.TransparencyMapFileName())
#            
#            if m.BumpMapFileName() != None:
#                if not m.BumpMapFileName() in self.imageFiles:
#                    self.imageFiles.append(m.BumpMapFileName())
                                                    
    def EventPropChange(self, e):
        n = self.propCB.GetValue()
        found = False
        for f in poser.Scene().Figures():
            if f.Name() == n:
                self.currentPropFigure = f
                self.fillMaterialsList(f)
                found = True
                break
        if not found:
            for p in poser.Scene().Actors():
                if p.IsProp():
                    if p.Name() == n:
                        self.fillMaterialsList(p)
                        self.currentPropFigure = p
                        break
                        
    def EventTextCtrlKey(self, e): 
        keycode = e.GetKeyCode()
        try:
            key=chr(keycode)           
        except:
            key = 'aaa'
        if key == 'aaa' or (key >= '0' and key <= '9') or keycode == 8 or keycode == 127: # 0-9, backspace, del
            e.Skip()

    def validateTextBox(self, textCtrl):
        try:
            if float(textCtrl.GetValue()) < 1:
                textCtrl.SetValue("1")
            if float(textCtrl.GetValue()) > 99:
                textCtrl.SetValue(str(99))
        except:
            textCtrl.SetValue("50")

    def printMem(self):
        m = self.calcTotalMemoryRequired()
        self.totalMemoryStaticText.SetLabel(str(int(m / 1024 / 1024)) + "MB")
                
    def EventCompress(self, e):
        self.SetCursor(wx.StockCursor(wx.CURSOR_WAIT))
        convertedList = {}
        self.validateTextBox(self.compressionTextCtrl)
        compress = int(self.compressionTextCtrl.GetValue())
        
        for i in range(self.checkListBox.GetCount()):
            if self.checkListBox.IsChecked(i):
                fname = self.checkListBox.GetClientData(i)
                if string.rfind(fname, "\\") != -1:
                    fname = fname[string.rfind(fname, "\\") + 1:]
                if string.rfind(fname, "/") != -1:
                    fname = fname[string.rfind(fname, "/") + 1:]
                
                # remove the file extension
                if string.rfind(fname, ".") != -1:
                    fname = fname[:string.rfind(fname, ".")]
                    
                saveF = COMPRESSED_STORAGE_FOLDER + "\\" + fname + str(compress) + ".jpg"
                nolog = wx.LogNull() 
                image = wx.Image(self.checkListBox.GetClientData(i))
                del nolog 
                image.Rescale (image.GetWidth() * compress / 100, image.GetHeight() * compress / 100)
                image.SaveFile(saveF, wx.BITMAP_TYPE_JPEG)
                convertedList[self.checkListBox.GetClientData(i)] = saveF

        for mat in self.currentPropFigure.Materials():
            for node in mat.ShaderTree().Nodes():
                if node != None:
                    if node.Type() == "image_map":
                        textureFilename = node.InputByInternalName("Image_Source").Value()
                        if textureFilename in convertedList:
                            node.InputByInternalName("Image_Source").SetString(convertedList[textureFilename]) 
                
            mat.ShaderTree().UpdatePreview()
        
        poser.Scene().DrawAll()            
        
        self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
        self.EndModal(1)
        
    def EventClose(self, e):
        finished = True
        self.EndModal(0)

    def printImage(self, fName, type):
        nolog = wx.LogNull() 
        image = wx.Image(fName)
        del nolog 
        fullName = fName
        size = float(max(image.GetWidth(), image.GetHeight()))
        if type == "Bump" or type == "Trans":
            size = size * size * 1.0
            if fullName not in self.GlobalGrayscaleFileList:
                self.GlobalGrayscaleFileList.append(fullName)
                self.TotalMemory += size
        else:    
            size = size * size * 4.0
            if fullName not in self.GlobalRGBFileList:
                self.GlobalRGBFileList.append(fullName)
                self.TotalMemory += size
        
    def scanForTexture(self, node, type):
        if node != None:
            if node.Type() == "image_map":
                texturename = node.InputByInternalName("Image_Source").Value()
                if type == "diffuse":
                    if texturename not in self.textureList:
                        self.textureList.append(texturename)
                        self.imageFiles.append(texturename)
                        self.printImage(texturename, "Diffuse")
                elif type == "bump":
                    if texturename not in self.bumpList:
                        self.bumpList.append(texturename)
                        self.imageFiles.append(texturename)
                        self.printImage(texturename, "Bump")
        
            for inputs in node.Inputs():
                if inputs.InNode() != None:
                    self.scanForTexture(inputs.InNode(), type)
                
    def calcTotalMemoryRequired(self):
        self.SetCursor(wx.StockCursor(wx.CURSOR_WAIT))
        
        TotalTextureMaps = 0
        TotalBumpMaps = 0
        self.TotalMemory = 0.0
        
        self.GlobalRGBFileList = []
        self.GlobalGrayscaleFileList = []       
        
        for figure in poser.Scene().Figures():
            if figure.Visible() == 1:
                self.textureList = []
                self.bumpList = []
                for material in figure.Materials():
                                
                    # Scan for the DIFFUSE map
                    node = material.ShaderTree().Nodes()[0].InputByInternalName("Diffuse_Color").InNode()
                    self.scanForTexture(node, "diffuse")
                    node = material.ShaderTree().Nodes()[0].InputByInternalName("AlternateDiffuse").InNode()
                    self.scanForTexture(node, "diffuse")

                    # Scan for the BUMP map
                    node = material.ShaderTree().Nodes()[0].InputByInternalName("Bump").InNode()
                    self.scanForTexture(node, "bump")

                    # Scan for the TRANS map
                    node = material.ShaderTree().Nodes()[0].InputByInternalName("Transparency_Max").InNode()
                    self.scanForTexture(node, "bump")

#                    if material.TextureMapFileName() != None:
#                        if material.TextureMapFileName() not in textureList:
#                            textureList.append(material.TextureMapFileName())
#                            self.printImage(material.TextureMapFileName(), "Diffuse")
#                    if material.BumpMapFileName() != None:
#                        if material.BumpMapFileName() not in bumpList:
#                            bumpList.append(material.BumpMapFileName())            
#                            self.printImage(material.BumpMapFileName(), "Bump")
#                    if material.TransparencyMapFileName() != None:
#                        if material.TransparencyMapFileName() not in textureList:
#                            textureList.append(material.TransparencyMapFileName())
#                            self.printImage(material.TransparencyMapFileName(), "Trans")
                numBumpMaps = 0
                for m in self.bumpList:
                    if m not in self.textureList:
                        numBumpMaps += 1
                TotalTextureMaps += len(self.textureList)
                TotalBumpMaps += numBumpMaps            
            
        for prop in poser.Scene().Actors():
            if prop.IsProp():
                if prop.Visible() == 1:
                    if prop.IsDeformer() == 0 and prop.IsBase() == 0 and prop.IsZone() == 0 and prop.IsLight() == 0 and prop.Geometry() != None and prop.IsCamera() == 0 and prop.Visible() == 1 and prop.Name() not in ["FocusDistanceControl", "GoalCenterOfMass"]:
                        self.textureList = []
                        self.bumpList = []
                        for material in prop.Materials():
                            # Scan for the DIFFUSE map
                            node = material.ShaderTree().Nodes()[0].InputByInternalName("Diffuse_Color").InNode()
                            self.scanForTexture(node, "diffuse")
                            node = material.ShaderTree().Nodes()[0].InputByInternalName("AlternateDiffuse").InNode()
                            self.scanForTexture(node, "diffuse")
        
                            # Scan for the BUMP map
                            node = material.ShaderTree().Nodes()[0].InputByInternalName("Bump").InNode()
                            self.scanForTexture(node, "bump")
        
                            # Scan for the TRANS map
                            node = material.ShaderTree().Nodes()[0].InputByInternalName("Transparency_Max").InNode()
                            self.scanForTexture(node, "bump")
                        
#                            shaderTree = material.ShaderTree()
#                            if material.TextureMapFileName() != None:
#                                if material.TextureMapFileName() not in textureList:
#                                    textureList.append(material.TextureMapFileName())
#                                    self.printImage(material.TextureMapFileName(), "Diffuse")
#                            if material.BumpMapFileName() != None:
#                                if material.BumpMapFileName() not in bumpList:
#                                    bumpList.append(material.BumpMapFileName())
#                                    self.printImage(material.BumpMapFileName(), "Bump")            
#                            if material.TransparencyMapFileName() != None:
#                                if material.TransparencyMapFileName() not in textureList:
#                                    textureList.append(material.TransparencyMapFileName())
#                                    self.printImage(material.TransparencyMapFileName(), "Trans")
                        numBumpMaps = 0
                        for m in self.bumpList:
                            if m not in self.textureList:
                                numBumpMaps += 1
                        TotalTextureMaps += len(self.textureList)
                        TotalBumpMaps += numBumpMaps

        self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
        return self.TotalMemory

LoadSettings()   
COMPRESSED_STORAGE_FOLDER = m_LiveDbTexturemapFolder
#COMPRESSED_STORAGE_FOLDER = poser.PrefsLocation()
finished = False
w = CompressTexturemaps()
w.ShowModal()

