# OctaneRender for Poser: Create Emitter From Lights
# 
# Copyright (c) 2012, Paul Kinnane, Physicalc Software Pty Ptd
#
# For additional support, contact Paul Kinnane at support@physicalc-software.com
#
# Version 1.0 - original release
#
# Version 1.02q - infinite lights now no longer converted - since they will feed the Octane dayilght system instead
#
#========================================================================================================================================
import poser

light = poser.Scene().CurrentActor()
if light.IsLight():
    if light.LightOn() == True:
        emitterName = "emitterprop_" + light.Name()
        if light.LightType() == poser.kLightCodeIMAGE:
            print "ERROR: This script requires a point or spot light to convert"
        elif light.LightType() == poser.kLightCodeINFINITE:
            print "ERROR: This script requires a point or spot light to convert"
        elif light.LightType() == poser.kLightCodePOINT:
            print "Creating emitter prop for point light : " + light.Name()
            poser.Scene().LoadLibraryProp("/Runtime/Libraries/Props/Primitives/Ball.ppz")
            newProp = poser.Scene().CurrentActor()
            newProp.SetName(emitterName)
            newProp.SetParent(light, 0, 1)
            newProp.ParameterByCode (poser.kParmCodeXTRAN).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeZTRAN).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeYTRAN).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeXROT).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeZROT).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeYROT).SetValue(0)
            r, g, b = light.ParameterByCode (poser.kParmCodeKDRED).Value(), light.ParameterByCode (poser.kParmCodeKDGREEN).Value(), light.ParameterByCode (poser.kParmCodeKDBLUE).Value()
            intensity = (r + g + b) / 3 * light.ParameterByCode (poser.kParmCodeKDINTENSITY).Value()
            material = newProp.Material("Preview")
            shaderTree = material.ShaderTree()
            shaderTree.Nodes()[0].InputByInternalName("Ambient_Color").SetColor(1, 1, 1)
            shaderTree.Nodes()[0].InputByInternalName("Ambient_Value").SetFloat(intensity * 300)
            shaderTree.UpdatePreview()
            print "Finished"            
        elif light.LightType() == poser.kLightCodeSPOT:
            print "Creating emitter prop for spot light : " + light.Name()
            poser.Scene().LoadLibraryProp("/Runtime/Libraries/Props/Primitives/one sided square.ppz")            
            newProp = poser.Scene().CurrentActor()
            newProp.SetName(emitterName)
            newProp.SetParent(light, 0, 1)
            newProp.ParameterByCode (poser.kParmCodeXTRAN).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeZTRAN).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeYTRAN).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeXROT).SetValue(180)
            newProp.ParameterByCode (poser.kParmCodeZROT).SetValue(0)
            newProp.ParameterByCode (poser.kParmCodeYROT).SetValue(0)
            r, g, b = light.ParameterByCode (poser.kParmCodeKDRED).Value(), light.ParameterByCode (poser.kParmCodeKDGREEN).Value(), light.ParameterByCode (poser.kParmCodeKDBLUE).Value()
            intensity = (r + g + b) / 3 * light.ParameterByCode (poser.kParmCodeKDINTENSITY).Value()
            material = newProp.Material("Preview")
            shaderTree = material.ShaderTree()
            shaderTree.Nodes()[0].InputByInternalName("Ambient_Color").SetColor(1, 1, 1)
            shaderTree.Nodes()[0].InputByInternalName("Ambient_Value").SetFloat(intensity * 300)
            shaderTree.UpdatePreview()
            print "Finished"            
else:
    print "ERROR: You must select a light prior to running this script"
    
poser.Scene().DrawAll()
